from popcorn.models.AlsoWatched import AlsoWatched
from popcorn.models.Rank import Rank


class Content:
    text: str
    ranks: list[Rank]
    also_watched: list[AlsoWatched]

    def __init__(
        self, text: str, ranks: list[Rank], also_watched: list[AlsoWatched]
    ) -> None:
        self.text = text
        self.ranks = ranks
        self.also_watched = also_watche



from rich import print
from rich.columns import Columns
from rich.console import Group
from rich.panel import Panel
from rich.table import Table
from rich.tree import Tree

from popcorn.api.models.SearchApi import Item
from popcorn.models.ItemMovie import Person
from popcorn.utils import format_date

works = {
    0: "Director",
    1: "Person",
    2: "Producer",
    3: "Executive Producer",
    4: "Writer",
    5: "Composer",
    6: "Creator",
    7: "-- -",
    8: "-- -",
    9: "-- -",
    10: "-- -",
}


def people(peoples: list[Person]) -> Columns:
    def chooseType(idx: int, person: Person) -> str:
        return (
            createProduction(idx, person)
            if person["role_type"] != 1
            else createActor(idx, person)
        )

    directory = [
        Panel(chooseType(idx, people), expand=True)
        for idx, people in enumerate(peoples)
    ]
    return Columns(directory)


def createProduction(idx: int, person: Person) -> str:
    return f"#{idx + 1}\n[b][green]{person['name']}[/b]\n[red1][i]{works[person['role_type']]}[/i]\n:link: {__create_link(person)}"


def createActor(idx: int, person: Person) -> str:
    return f"#{idx+ 1 }\n[b][purple]{person['name']}[/b]\n[red1][i]{person['role']}[/i]\n:link: {__create_link(person)}"


def __create_link(person: Person) -> str:
    return (
        f"[blue1][link=https://reelgood.com/person/{person['slug']}]For Details[/link]"
    )


def people_biography(biography: str | None) -> Panel:
    return Panel(biography)


def person_media(list: list[Item]) -> Columns:
    def chooseType(idx: int, item: Item) -> str:
        return f"#{idx + 1}\n[red]{item['title']} \n[green]{format_date(item['released_on']).date()}\n[yellow]{item['imdb_rating']}"

    directory = [
        Panel(chooseType(idx, item), expand=True) for idx, item in enumerate(list)
    ]
    return Columns(directory)



from datetime import datetime

from popcorn.config import URL_BASE
from popcorn.models.Trailer import Trailer


def format_date(date: str | None) -> datetime | str | None:
    if date is None:
        return None
    try:
        formatDate = date[:19]
        return datetime.strptime(formatDate, "%Y-%m-%dT%H:%M:%S")
    except:
        return None


def format_date_str(date: str | None) -> str:
    if date is None:
        return "--"

    try:
        formatDate = date[:19]
        return str(datetime.strptime(formatDate, "%Y-%m-%dT%H:%M:%S").date())
    except:
        return "-- -- --"


def create_url(type: str, url: str) -> str:
    if type == "m":
        return URL_BASE + "movie" + "/" + url
    else:
        return URL_BASE + "show" + "/" + url


def format_type(type: chr) -> str:
    if type == "m":
        return "Movie"
    elif type == "s":
        return "Show"
    else:
        return "- -"


def format_source(item: str) -> str:
    return item.replace("_", " ").title()


def format_sources(sources: list[str]) -> str:
    result = ""
    br = has_br(sources.__len__())
    sources = remove_dash(sources=sources)
    for index, source in enumerate(sources):
        result = result + source + (br if index != sources.__len__() - 1 else "")
    return result


def remove_dash(sources: list[str]) -> list[str]:
    result = []
    for source in map(lambda x: x.replace("_", " ").capitalize(), sources):
        result.append(source)
    return result


def has_br(length: int):
    if length >= 15:
        return "\n"
    else:
        return " - "


def format_trailers(trailers: list[Trailer]) -> str:
    result = ""
    values = set(map(lambda x: x["key"], trailers))
    for index, value in enumerate(values):
        result = (
            result
            + f":link: [bold blue][link=https://youtu.be/{value}]Trailer - {index + 1}[/link][/bold blue]\n"
        )
    return result


def filterTrailerByService(trailers: list[Trailer], service="youtube") -> list[Trailer]:
    return filter(lambda x: x["site"] == service, trailers)



from rich.table import Table, box

from popcorn.api.models.PersonApi import PersonApi
from popcorn.utils import format_date_str

topTablePerson = [
    ":scroll: Name",
    ":date: Birth date",
    ":star: Birth place",
    ":date: Death date",
]


def initTable() -> Table:
    table = Table(
        highlight=True,
        show_header=True,
        show_edge=True,
        expand=True,
        show_lines=True,
        box=box.DOUBLE_EDGE,
    )

    for title in topTablePerson:
        table.add_column(title)
    return table


def __tableItemPerson__(
    table: Table, name: str, birthdate: str, birthplace: str, deathdate: str
) -> Table:
    table.add_row(name, birthdate, birthplace, deathdate)
    return table


def table_details_person(item: PersonApi) -> Table:
    table = __tableItemPerson__(
        table=initTable(),
        name=item.name,
        birthdate="-- -- --"
        if item.birthdate is None
        else format_date_str(item.birthdate),
        birthplace="" if item.birthplace is None else item.birthplace,
        deathdate="-- -- --"
        if item.deathdate is None
        else format_date_str(item.deathdate),
    )
    return table



from datetime import datetime

import typer
from rich.console import Console
from rich.panel import Panel
from rich.prompt import Prompt
from rich.table import Table
from rich.tree import Tree

from popcorn.api.models.PersonApi import PersonApi
from popcorn.api.models.Result import Result
from popcorn.api.models.SearchApi import Item
from popcorn.controller import person_reel, search_reel, transform_item
from popcorn.models.ItemMovie import ItemMovie
from popcorn.models.ItemShow import ItemShow
from popcorn.models.Person import Person
from popcorn.models.Search import Search
from popcorn.tables.people import people, people_biography, person_media
from popcorn.tables.sources import collumn_seasons, pop_collumns
from popcorn.tables.table_details_movie import table_details
from popcorn.tables.table_person import table_details_person
from popcorn.tables.table_search import table_search
from popcorn.tables.tableDetailsShow import table_details_show
from popcorn.utils import format_date

app = typer.Typer(help="PopCorn Show")
console = Console()


@app.command(name="search")
def search(
    name: str,
    year: int = typer.Option(None, "--year", "-y", help="type Year"),
    type: str = typer.Option(None, "--type", "-t", help="'m' or 's'"),
    luck: bool = typer.Option(False, "--luck", "-l"),
):
    list = search_reel(name, year=year, type=type)
    if list:
        if luck:
            order = sorted(list, key=lambda x: x.imdb_str(), reverse=True)
            result = transform_item(order[0].slug, order[0].type)
            choose_types(result)
        else:
            console.print(table_search(list))
            choose_number(list)
    else:
        typer.echo("Not Found")


def person(name: str):
    result: Result = person_reel(name)
    if result.error is None:
        __show_person(result.value)
    else:
        console.print(result.error)


def choose_types(result: Result):
    if result.error:
        console.print(result.error)
    else:
        fill_by_type(result)


def fill_by_type(result: Result):
    if isinstance(result.value, ItemMovie):
        __show_movie(result.value)
    elif isinstance(result.value, ItemShow):
        __show_tvshow(result.value)


def __show_tvshow(show: ItemShow):
    tree = Tree("")
    tree.add(__rich__())
    details = Tree(
        f":blue_book: [blue][b]Details - {show.title} {format_date(show.released_on).year}[/b] - :link: [blue][link={show.createUrl()}]Details in Reelgood.com[/link]",
        expanded=True,
    )
    details.add(table_details_show(show), highlight=False)

    person = Tree(":busts_in_silhouette:[bold][purple] People")
    person.add(people(show.people))

    tree.add(details)
    if person.children.__len__() > 0:
        tree.add(person)
    if show.seasons.__len__() > 0:
        tv = Tree(
            f":movie_camera: [red][b]Where to Watch: {show.title} [/red]| [yellow]With Season {show.seasons.__len__()}"
        )
        tv.add(
            collumn_seasons(show.episodes, reversed(show.seasons)),
            expanded=True,
            highlight=False,
        )
        tree.add(tv)
    tree.add(__footer__(show))
    console.print(tree)

    if show.people.__len__() > 0:
        __choose_person(show.people)


def __show_movie(movie: ItemMovie):
    tree = Tree("")
    tree.add(__rich__())
    details = Tree(
        f":blue_book:[blue][b] Details - {movie.title} {format_date(movie.released_on).year if format_date(movie.released_on) != None else '-- --'}[/b] - :link: [blue][link={movie.create_url()}]Details in Reelgood.com[/link]",
        expanded=True,
    )
    details.add(table_details(movie), highlight=False)

    person = Tree(":busts_in_silhouette:[bold][purple] People")
    person.add(people(movie.people))

    tree.add(details)
    if person.children.__len__() > 0:
        tree.add(person)
    if movie.availability:
        tv = Tree(f":movie_camera:[red][b] Where to Watch: {movie.title}")
        tv.add(pop_collumns(movie.availability), expanded=True, highlight=False)
        tree.add(tv)
    tree.add(__footer__(movie))
    console.print(tree)
    if movie.people.__len__() > 0:
        __choose_person(movie.people)


def __choose_person(list: list[Person]):
    number = Prompt.ask(
        "[blue]Enter the person's number between 1 to "
        + f"{1 if list.__len__() == 1 else list.__len__()} "
        + "for details - Zero to exit\n"
    )
    try:
        value = int(number)
        if value < 1:
            return
        people = list[value - 1]
        person(people["slug"])
        return
    except (TypeError, ValueError, IndexError):
        __choose_person(list)


def __show_person(person: PersonApi):
    tree = Tree("")
    tree.add(__rich__())
    tree.add(table_details_person(person), highlight=False)
    if person.biography is not None:
        tree.add(people_biography(person.biography))
    if person.initial_credits.__len__() > 0:
        tree.add(person_media(person.initial_credits))
    tree.add(__footer_person__(person))
    console.print(tree)

    if person.initial_credits.__len__() > 0:
        choose_media_by_person(person)


def choose_media_by_person(person: PersonApi):
    length = person.initial_credits.__len__()
    number = Prompt.ask(
        "[red]Enter the media number 1 to "
        + f"{1 if length == 1 else length} "
        + "[red]for details - Zero to exit\n"
    )
    try:
        value = int(number)
        if value < 1:
            return
        item: Item = person.initial_credits[value - 1]
        media: Result = transform_item(item["slug"], item["content_type"])
        choose_types(media)
    except (IndexError, ValueError):
        choose_media_by_person(person)


def __rich__() -> Panel:
    grid = Table.grid(expand=True)
    grid.add_column(justify="center", ratio=1)
    grid.add_column(justify="right")
    grid.add_row(
        ":corn::corn::corn:[b] POPCORN SHOW - CLI[/b]:corn::corn::corn:",
        datetime.now().ctime().replace(":", "[blink]:[/]"),
    )
    return Panel(grid, style="red1 on black")


def __footer__(item: ItemMovie | ItemShow) -> Panel:
    grid = Table.grid(expand=True)
    grid.add_column(justify="left", ratio=3)
    grid.add_column(justify="right", ratio=1)
    if item.score_breakdown["content"]["text"]:
        grid.add_row(
            f"[orange1][b]{item.score_breakdown['content']['text']}",
            f"[b][link=https://play.google.com/store/apps/details?id=br.com.icaro.filme][yellow]App Android => [/link]",
        )
    return Panel(grid, style="red1 on black")


def __footer_person__(item: PersonApi) -> Panel:
    grid = Table.grid(expand=True)
    if item.homepage:
        grid.add_column(justify="left", ratio=3)
        grid.add_column(justify="right", ratio=1)
        link = f"[b][orange1]:link:[link={item.homepage}] Site"
        grid.add_row(
            link,
            f"[b][link=https://play.google.com/store/apps/details?id=br.com.icaro.filme][yellow]App Android => [/link]",
        )
    else:
        grid.add_column(justify="right", ratio=1)
        grid.add_row(
            "",
            f"[b][link=https://play.google.com/store/apps/details?id=br.com.icaro.filme][yellow]App Android => [/link]",
        )
    return Panel(grid, style="red1 on black")


def choose_number(list: list[Search], hasError=False):
    item: None
    number = Prompt.ask(
        f"[red]For details, write a number between 1 and {list.__len__()}. Zero to exit\n"
    )
    if hasError or number == "0":
        return

    try:
        number = int(number)
        if number > 0 and number <= list.__len__():
            item = list[number - 1]
        else:
            print("Number off the list.")
            choose_number(list, True)
            return

    except ValueError:
        print("Number Error! - write '0' to exit ")
        choose_number(list, True)
        return

    if item is not None:
        result = transform_item(item.slug, item.type)
        fill_by_type(result)



from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import Any, List, Optional

""""
    LIMPAR CLASSE
"""


class Classification(Enum):
    THE_13 = "13+"
    THE_16 = "16+"
    THE_18 = "18+"
    THE_7 = "7+"


class ContentType(Enum):
    def __str__(self) -> str:
        return self.name

    M = "m"
    S = "s"


@dataclass
class Item:
    id: str
    slug: str
    content_type: ContentType
    title: str
    overview: None
    availability_pros: None
    imdb_rating: Optional[float]
    rt_critics_rating: None
    rg_content_score: None
    has_poster: bool
    poster_blur: None
    has_backdrop: bool
    backdrop_blur: None
    released_on: Optional[datetime]
    classification: Optional[Classification]
    sources: List[str]
    genres: List[Any]
    on_services: bool
    on_free: bool
    on_rent_purchase: bool
    user_rating: None
    tracking: bool
    watchlisted: bool
    seen: bool
    season_count: int
    featured_services: List[Any]
    episode_source_count: int

    def __init__(
        self,
        id: str,
        slug: str,
        content_type: ContentType,
        title: str,
        overview: None,
        availability_pros: None,
        imdb_rating: Optional[float],
        rt_critics_rating: None,
        rg_content_score: None,
        has_poster: bool,
        poster_blur: None,
        has_backdrop: bool,
        backdrop_blur: None,
        released_on: Optional[datetime],
        classification: Optional[Classification],
        sources: List[str],
        genres: List[Any],
        on_services: bool,
        on_free: bool,
        on_rent_purchase: bool,
        user_rating: None,
        tracking: bool,
        watchlisted: bool,
        seen: bool,
        season_count: int,
        featured_services: List[Any],
        episode_source_count: int,
    ) -> None:
        self.id = id
        self.slug = slug
        self.content_type = content_type
        self.title = title
        self.overview = overview
        self.availability_pros = availability_pros
        self.imdb_rating = imdb_rating
        self.rt_critics_rating = rt_critics_rating
        self.rg_content_score = rg_content_score
        self.has_poster = has_poster
        self.poster_blur = poster_blur
        self.has_backdrop = has_backdrop
        self.backdrop_blur = backdrop_blur
        self.released_on = released_on
        self.classification = classification
        self.sources = sources
        self.genres = genres
        self.on_services = on_services
        self.on_free = on_free
        self.on_rent_purchase = on_rent_purchase
        self.user_rating = user_rating
        self.tracking = tracking
        self.watchlisted = watchlisted
        self.seen = seen
        self.season_count = season_count
        self.featured_services = featured_services
        self.episode_source_count = episode_source_count


@dataclass
class SearchApi:
    items: list[Item]
    page: int
    pages: int
    page_size: int

    #  FIZ PARA USAR LISTA
    def __init__(
        self, items: list[Item], page: int, pages: int, page_size: int
    ) -> None:
        print(items)
        self.items = items
        self.page = page
        self.pages = pages
        self.page_size = page_size



class TVShowEpisode:
    def __init__(self, data):
        self.id = data["id"]
        self.season_id = data["season_id"]
        self.title = data["title"]
        self.overview = data["overview"]
        self.number = data["number"]
        self.sequence_number = data["sequence_number"]
        self.watched = data["watched"]
        self.has_screenshot = data["has_screenshot"]
        self.aired_at = data["aired_at"]
        self.availability = []
        for a in data["availability"]:
            av = Availability(
                a["rental_cost_hd"],
                a["rental_cost_sd"],
                a["purchase_cost_hd"],
                a["purchase_cost_sd"],
                a["source_id"],
                a["source_name"],
                a["access_type"],
                SourceData(
                    a["source_data"]["links"]["web"],
                    a["source_data"]["links"]["ios"],
                    a["source_data"]["links"]["android"],
                    References(
                        a["source_data"]["references"]["web"]["episode_id"],
                        a["source_data"]["references"]["web"]["show_id"],
                        a["source_data"]["references"]["ios"]["episode_id"],
                        a["source_data"]["references"]["ios"]["show_id"],
                        a["source_data"]["references"]["android"]["episode_id"],
                        a["source_data"]["references"]["android"]["show_id"],
                    ),
                    a["source_data"]["web_link"],
                ),
            )
            self.availability.append(av)


class Availability:
    def __init__(
        self,
        rental_cost_hd,
        rental_cost_sd,
        purchase_cost_hd,
        purchase_cost_sd,
        source_id,
        source_name,
        access_type,
        source_data,
    ):
        self.rental_cost_hd = rental_cost_hd
        self.rental_cost_sd = rental_cost_sd
        self.purchase_cost_hd = purchase_cost_hd
        self.purchase_cost_sd = purchase_cost_sd
        self.source_id = source_id
        self.source_name = source_name
        self.access_type = access_type
        self.source_data = source_data


class SourceData:
    def __init__(self, web, ios, android, references, web_link):
        self.links = Links(web, ios, android)
        self.references = references
        self.web_link = web_link


class Links:
    def __init__(self, web, ios, android):
        self.web = web
        self.ios = ios
        self.android = android


class References:
    def __init__(
        self,
        web_episode_id,
        web_show_id,
        ios_episode_id,
        ios_show_id,
        android_episode_id,
        android_show_id,
    ):
        self.web = WebReferences(web_episode_id, web_show_id)
        self.ios = IOSReferences(ios_episode_id, ios_show_id)
        self.android = AndroidReferences(android_episode_id, android_show_id)


class WebReferences:
    def __init__(self, episode_id, show_id):
        self.episode_id = episode_id
        self.show_id = show_id


class IOSReferences:
    def __init__(self, episode_id, show_id):
        self.episode_id = episode_id
        self.show_id = show_id


class AndroidReferences:
    def __init__(self, episode_id, show_id):
        self.episode_id = episode_id
        self.show_id = show_id



from popcorn.models.Creative import Creative
from popcorn.models.Links import Links


class SourceAdPlacement:
    placement_type: str
    source_id: str
    source_name: str
    access_type: int
    image_url: str
    title_copy: str
    short_deal_copy: str
    long_deal_copy: str
    additional_copy: None
    button_call_to_action: str
    landing_page_link: None
    use_deeplink: bool
    deeplink: str
    inline_position: str
    min_price: float
    creative: Creative
    links: Links

    def __init__(
        self,
        placement_type: str,
        source_id: str,
        source_name: str,
        access_type: int,
        image_url: None,
        title_copy: str,
        short_deal_copy: str,
        long_deal_copy: str,
        additional_copy: None,
        button_call_to_action: str,
        landing_page_link: None,
        use_deeplink: bool,
        deeplink: str,
        inline_position: str,
        min_price: float,
        creative: Creative,
        links: Links,
    ) -> None:
        self.placement_type = placement_type
        self.source_id = source_id
        self.source_name = source_name
        self.access_type = access_type
        self.image_url = image_url
        self.title_copy = title_copy
        self.short_deal_copy = short_deal_copy
        self.long_deal_copy = long_deal_copy
        self.additional_copy = additional_copy
        self.button_call_to_action = button_call_to_action
        self.landing_page_link = landing_page_link
        self.use_deeplink = use_deeplink
        self.deeplink = deeplink
        self.inline_position = inline_position
        self.min_price = min_price
        self.creative = creative
        self.links = links



from typing import Optional


class Links:
    web: str
    ios: Optional[str]
    android: Optional[str]

    def __init__(self, web: str, ios: Optional[str], android: Optional[str]) -> None:
        self.web = web
        self.ios = ios
        self.android = android



class ListingKey:
    listing_type: str
    listing_identifier: str

    def __init__(self, listing_type: str, listing_identifier: str) -> None:
        self.listing_type = listing_type
        self.listing_identifier = listing_identifier



from popcorn.api.models.SearchApi import Item
from popcorn.models.Search import Search


def format_list(items: list[Item]) -> list[Search]:
    def dto(values):
        index, item = values
        return Search(
            id=index,
            title=item["title"],
            imdb=item["imdb_rating"],
            type=item["content_type"],
            release=item["released_on"],
            online=item["sources"],
            slug=item["slug"],
        )

    result = map(lambda item: dto(item), enumerate(items))
    return sorted(result, key=lambda item: str(item.release), reverse=True)



from rich.columns import Columns
from rich.panel import Panel
from rich.tree import Tree

from popcorn.models.ItemMovie import Availability
from popcorn.models.Seasons import Availability as AvailabilityShow
from popcorn.models.Seasons import Season
from popcorn.models.TVShowEpisode import TVShowEpisode
from popcorn.utils import format_source


def pop_collumns(availabilitys: list[Availability]) -> Columns:
    return Columns(
        [Panel(panel, expand=True, safe_box=True) for panel in source(availabilitys)]
    )


def collumn_seasons(episodes: list[TVShowEpisode], season: list[Season]) -> Columns:
    return Columns(
        [
            Panel(panel, expand=True, safe_box=True)
            for panel in sourceSeason(episodes, season)
        ]
    )


def source(availabilitys: list[Availability]) -> list[Tree]:
    result = []
    for availability in availabilitys:
        source = Tree(__getTitle(availability=availability))

        # removido node com valores - adicionado na linha de titulo
        # rental = __getRentalTree(availability=availability)
        # purchase = __getPurchaseTree(availability=availability)

        # if purchase.children.__len__() > 0:
        #     source.add(purchase)
        # if rental.children.__len__() > 0:
        #     source.add(rental)

        result.append(source)
    return result


def sourceSeason(episodes: list[TVShowEpisode], seasons: list[Season]) -> list[Tree]:
    result = []
    for season in seasons:
        ep = first_ep_for_season(episodes, season)
        source = Tree(
            __getListSourceForSeason(ep, season["number"], season["availability"])
        )
        result.append(source)
    return result


def first_ep_for_season(
    episodes: list[TVShowEpisode], season: Season
) -> TVShowEpisode | None:
    for eps in season["episodes"]:
        if eps:
            try:
                ep = episodes[eps]
                if ep["number"] == 1:
                    return ep
            except:
                return None
        else:
            return None


def __getTitle(availability: Availability) -> str:
    if (
        "source_data" in availability
        and "web_link" in availability["source_data"]
        and "source_name" in availability
    ):
        return f"[b]{format_source(availability['source_name'])}[/b] {__minAndMax(availability)} :link: [i][blue]{availability['source_data']['web_link']}[/i]"
    else:
        return f"[b]{availability['source_name']}[/b]"


def __getListSourceForSeason(
    ep: TVShowEpisode, numberSeason: int, availability: AvailabilityShow
) -> str:
    result = f"Season {numberSeason} Availability | "
    for source in availability["sources"]:
        link = __fistLink(ep, source["source_name"])
        result = result + __createLinkForSeason(source["source_name"], link) + " "
    return result


def __fistLink(ep: TVShowEpisode, source_name: str) -> str:
    try:
        for av in ep["availability"]:
            if av["source_name"] == source_name:
                return av["source_data"]["web_link"]
        return ""
    except:
        return ""


def __createLinkForSeason(source_name: str, link: str) -> str:
    return f":link: [bold blue][link={link}]{format_source(source_name)}[/link][/bold blue]"


def __minAndMax(availability: Availability) -> str:
    values = []
    if "rental_cost_sd" in availability and availability["rental_cost_sd"] is not None:
        values.append(availability["rental_cost_sd"])
    if "rental_cost_hd" in availability and availability["rental_cost_hd"] is not None:
        values.append(availability["rental_cost_hd"])
    if (
        "purchase_cost_sd" in availability
        and availability["purchase_cost_sd"] is not None
    ):
        values.append(availability["purchase_cost_sd"])
    if (
        "purchase_cost_hd" in availability
        and availability["purchase_cost_hd"] is not None
    ):
        values.append(availability["purchase_cost_hd"])
    if values.__len__() == 0:
        return ""
    return f" | :dollar: {min(values)} :moneybag: {max(values)} |"


def __getRentalTree(availability: Availability) -> Tree:
    rental = Tree("Rental")
    if (
        "rental_cost_hd" in availability
        and "rental_cost_hd" in availability
        and availability["rental_cost_sd"] == availability["rental_cost_hd"]
    ):
        rental.add(f":moneybag: {availability['rental_cost_sd']} SD - HD")
    else:
        if "rental_cost_sd" in availability:
            rental.add(f":dollar: {availability['rental_cost_sd']} SD")
        if "rental_cost_hd" in availability:
            rental.add(f":moneybag: {availability['rental_cost_hd']} HD")
    return rental


def __getPurchaseTree(availability: Availability) -> Tree:
    purchase = Tree("Purchase")
    if (
        "purchase_cost_sd" in availability
        and "purchase_cost_hd" in availability
        and availability["purchase_cost_sd"] == availability["purchase_cost_hd"]
    ):
        purchase.add(f":moneybag: {availability['purchase_cost_hd']} SD - HD")
    else:
        if "purchase_cost_sd" in availability:
            purchase.add(f":dollar: {availability['purchase_cost_sd']} SD")
        if "purchase_cost_hd" in availability:
            purchase.add(f":moneybag: {availability['purchase_cost_hd']} HD")
    return purchase



from dataclasses import dataclass
from datetime import datetime
from typing import Any

from popcorn.models.Metadata import Metadata
from popcorn.models.Person import Person
from popcorn.models.RatingStats import RatingStats
from popcorn.models.ReelgoodScores import ReelgoodScores
from popcorn.models.RegionalAvailability import RegionalAvailability
from popcorn.models.ScoreBreakdown import ScoreBreakdown
from popcorn.models.Seasons import Season
from popcorn.models.SourceAdPlacement import SourceAdPlacement
from popcorn.models.Tag import Tag
from popcorn.models.Trailer import Trailer
from popcorn.models.TVShowEpisode import TVShowEpisode
from popcorn.tables.TableInterface import TableInterface
from popcorn.utils import create_url as full_url
from popcorn.utils import filterTrailerByService, format_date, format_trailers


@dataclass
class Android:
    episode_id: str

    @staticmethod
    def from_dict(obj: Any) -> "Android":
        _episode_id = str(obj.get("episode_id"))
        return Android(_episode_id)


@dataclass
class Ios:
    episode_id: str

    @staticmethod
    def from_dict(obj: Any) -> "Ios":
        _episode_id = str(obj.get("episode_id"))
        return Ios(_episode_id)


@dataclass
class Links:
    web: str
    ios: str
    android: str

    @staticmethod
    def from_dict(obj: Any) -> "Links":
        _web = str(obj.get("web"))
        _ios = str(obj.get("ios"))
        _android = str(obj.get("android"))
        return Links(_web, _ios, _android)


@dataclass
class References:
    # web: Web
    ios: Ios
    android: Android
    # webos: Webos

    @staticmethod
    def from_dict(obj: Any) -> "References":
        _web = Web.from_dict(obj.get("web"))
        _ios = Ios.from_dict(obj.get("ios"))
        _android = Android.from_dict(obj.get("android"))
        _webos = Webos.from_dict(obj.get("webos"))
        return References(_web, _ios, _android, _webos)


@dataclass
class SourceData:
    links: Links
    references: References
    web_link: str

    @staticmethod
    def from_dict(obj: Any) -> "SourceData":
        _links = Links.from_dict(obj.get("links"))
        _references = References.from_dict(obj.get("references"))
        _web_link = str(obj.get("web_link"))
        return SourceData(_links, _references, _web_link)


@dataclass
class Web:
    episode_id: str

    @staticmethod
    def from_dict(obj: Any) -> "Web":
        _episode_id = str(obj.get("episode_id"))
        return Web(_episode_id)


@dataclass
class Webos:
    episode_id: str

    @staticmethod
    def from_dict(obj: Any) -> "Webos":
        _episode_id = str(obj.get("episode_id"))
        return Webos(_episode_id)


@dataclass
class Recommended_episode:
    episode_id: str
    season_id: str


class TextReasons:
    text: str


class ItemShow(TableInterface):
    metadata: Metadata
    id: str
    slug: str
    title: str
    overview: str
    reelgood_synopsis: str
    trailer: Trailer
    trailers: list[Trailer]
    imdb_rating: float
    rt_critics_rating: None
    rt_audience_rating: int
    has_poster: bool
    has_backdrop: bool
    season_count: int
    classification: str
    last_modified_at: datetime
    sources: list[str]
    released_on: datetime
    user_rating: None
    seen: bool
    user_lists: None
    on_free: bool
    on_rent_purchase: bool
    genres: list[int]
    tags: list[Tag]
    countries: list[str]
    tracking: bool
    people: list[Person]
    completed_on: str
    score_breakdown: ScoreBreakdown
    regional_availability: RegionalAvailability
    reelgood_scores: ReelgoodScores
    returning_on: Any
    source_ad_placement: SourceAdPlacement
    source_ad_placements: list[SourceAdPlacement]
    unwatched: int
    coming_on: str
    has_new: bool
    recommended_episode: list[Recommended_episode]
    seasons: list[Season]
    episodes: list[TVShowEpisode]
    madlib_synopsis: str
    where_to_wathc: str
    reasons_to_watch: list[TextReasons]
    imdb_votes: int
    episode_availability: list[Any]
    content_blocks: any

    def __init__(
        self,
        metadata: Metadata,
        id: str,
        slug: str,
        title: str,
        overview: str,
        reelgood_synopsis: str,
        classification: str,
        released_on: datetime,
        trailer: Trailer,
        trailers: list[Trailer],
        has_poster: bool,
        has_backdrop: bool,
        imdb_rating: float,
        rt_critics_rating: None,
        rt_audience_rating: int,
        last_modified_at: datetime,
        user_rating: None,
        user_lists: None,
        sources: list[str],
        on_free: bool,
        on_rent_purchase: bool,
        genres: list[int],
        tags: list[Tag],
        countries: list[str],
        people: list[Person],
        score_breakdown: ScoreBreakdown,
        reelgood_scores: ReelgoodScores,
        regional_availability: RegionalAvailability,
        source_ad_placement: SourceAdPlacement,
        source_ad_placements: list[SourceAdPlacement],
        season_count: int,
        tracking: bool,
        completed_on: str,
        returning_on: Any,
        unwatched: int,
        coming_on: str,
        has_new: bool,
        recommended_episode: list[Recommended_episode],
        imdb_votes: int,
        seasons: list[Season],
        episodes: list[TVShowEpisode],
        episode_availability: list[Any],
        rating_stats: RatingStats,
        reviews_count: int = 0,
        madlib_synopsis: str = "",
        where_to_watch: str = "",
        reasons_to_watch: list[TextReasons] = [],
        content_blocks: str = "",
    ) -> None:
        self.metadata = metadata
        self.id = id
        self.slug = slug
        self.title = title
        self.overview = overview
        self.tags = tags
        self.reelgood_synopsis = reelgood_synopsis
        self.classification = classification
        self.released_on = released_on
        self.tags = tags
        self.trailer = trailer
        self.trailers = trailers
        self.has_poster = has_poster
        self.has_backdrop = has_backdrop
        self.imdb_rating = imdb_rating
        self.rt_audience_rating = rt_audience_rating
        self.rt_critics_rating = rt_critics_rating
        self.last_modified_at = last_modified_at
        self.user_rating = user_rating
        self.sources = sources
        self.on_free = on_free
        self.on_rent_purchase = on_rent_purchase
        self.genres = genres
        self.countries = countries
        self.people = people
        self.score_breakdown = score_breakdown
        self.regional_availability = regional_availability
        self.source_ad_placement = source_ad_placement
        self.source_ad_placements = source_ad_placements
        self.reelgood_scores = reelgood_scores
        self.season_count = season_count
        self.tracking = tracking
        self.completed_on = completed_on
        self.returning_on = returning_on
        self.unwatched = unwatched
        self.coming_on = coming_on
        self.has_new = has_new
        self.recommended_episode = recommended_episode
        self.seasons = seasons
        self.episodes = episodes
        self.rating_stats = rating_stats
        self.reviews_count = reviews_count
        self.madlib_synopsis = madlib_synopsis
        self.where_to_watch = where_to_watch
        self.reasons_to_watch = reasons_to_watch
        self.imdb_votes = imdb_votes
        self.episode_availability = episode_availability
        self.content_blocks = content_blocks

    def get_number_seasons(self):
        return str(self.season_count)

    def createUrl(self):
        return full_url("s", self.slug)

    def get_overview(self):
        return self.overview

    def get_date(self) -> str:
        return self.format_date()

    def get_time(self) -> str:
        return "-- --"

    def get_imdb(self) -> str:
        return str(self.imdb_rating)

    def get_classification(self) -> str:
        return (
            f"[b][white]{self.classification}" if self.classification else "Who knows?"
        )

    def get_trailers(self) -> str:
        return self.getListTrailers()

    def getListTrailers(self) -> str:
        if self.trailer is not None:
            self.trailers.append(self.trailer)
            return format_trailers(filterTrailerByService(self.trailers))
        else:
            search = self.slug.replace("-", "+")
            return f"[bold blue][link=https://www.youtube.com/results?search_query={search}]Search Youtube[/link]"

    def format_date(self) -> str:
        return (
            str(format_date(self.released_on).date())
            if self.released_on is not None
            else "-- --"
        )



import requests

from popcorn.api.models.Result import Result

paramsDefault = {
    "page": "1",
    "pageSize": "10",
    "region": "us",
    "take": "10",
    "terms": "",
}


def search(query: str) -> Result:
    """
    Busca por mídias com valores igual a 'query', retornando um json
    Parameters:
        query: texto nome do filme ou serie
    Returns:
        Retorna um object Result onde se tem objetos internos, como value contento o json do resultado
    Exemplos:
        >>> search('the matrix') -> Result

            "items": [{
                    "id": "2c947f66-ba1e-4747-91a3-ab1432f6bb96",
                    "slug": "the-lord-of-the-rings-the-return-of-the-king-2003",
                    "content_type": "m",
                    "title": "The Lord of the Rings: The Return of the King",
                    "overview": null,
                    "availability_pros": null,
                    "imdb_rating": 9,
                    "rt_critics_rating": null,
                    "rg_content_score": null,
                    "has_poster": true,
                    "poster_blur": null,
                    "has_backdrop": true,
                    "backdrop_blur": null,
                    "released_on": "2003-12-17T00:00:00Z",
                    "classification": "13+",
                    "sources": [
                        "hbo_max"
                    ],
                    "genres": [],
                    "on_services": false,
                    "on_free": false,
                    "on_rent_purchase": true,
                    "user_rating": null,
                    "tracking": false,
                    "watchlisted": false,
                    "seen": false,
                    "season_count": 0,
                    "featured_services": [],
                    "episode_source_count": 0
                    },
                    ...
                ]

        >>> search('um texto sem correspondência') -> Result

            {
               "items": [],
               "page": 1,
               "pages": 0,
               "page_size": 10
            }
    """
    paramsDefault["terms"] = query
    result = requests.get(
        "https://api.reelgood.com/v3.0/content/search/content", params=paramsDefault
    )
    if result.status_code == requests.codes.ok:
        return Result(value=result.text)
    else:
        return Result(error=Exception(result.content))


def get_movie_api(id: str) -> Result:
    """
    Busca por informações da filme, retornando um json
    Parameters:
        id: texto com ID completo de um filme
    Returns:
        Retorna um object Result onde se tem objetos internos, como value contento o json do resultado
    Exemplos:
        >>> getMovieApi('the matrix') -> Result

            {
                "metadata": {
                    "ad_data": null
                },
                "id": "6eb67f3f-113d-47e6-8776-406dda49f7ca",
                "slug": "the-matrix-1999",
                "title": "The Matrix",
                "overview": "When a beautiful stranger leads computer hacker Neo to a forbidding underworld, he discovers the shocking truth--the life he knows is the elaborate deception of an evil cyber-intelligence.",
                "tagline": "Welcome to the Real World.",
                "reelgood_synopsis": "The Matrix featuring Keanu Reeves and Laurence Fishburne is streaming on HBO MAX, streaming with subscription on fuboTV, streaming via tv everywhere with Syfy, and 8 others. It's an Action & Adventure and Science Fiction movie with a high IMDb audience rating of 8.7 (1,942,671 votes).",
                "classification": "18+",
                "runtime": 136,
                "released_on": "1999-03-31T00:00:00Z",
                "trailer": {
                    "site": "youtube",
                    "key": "nUEQNVV3Gfs",
                    "url": null
                },
                ...
            }
        >>> search('um texto sem correspondência')

            {
                "type": "https://tools.ietf.org/html/rfc7231#section-6.5.4",
                "title": "Not Found",
                "status": 404,
                "traceId": "00-879fe78e889ae66bb06aa8ae86c6ac11-9a132b9f1975f20e-00"
            }
    """
    result = requests.get(f"https://api.reelgood.com/v3.0/content/movie/{id}")
    if result.status_code == requests.codes.ok:
        return Result(value=result.text)
    else:
        return Result(error=Exception(result.content))


def get_tvshow_api(id: str) -> Result:
    """
    Busca por informações da série, retornando um json
    Parameters:
        id: texto com ID completo de um Série
    Returns:
        Retorna um object Result onde se tem objetos internos, como value contento o json do resultado
    Exemplos:
        >>> getTvShowApi('the matrix') -> Result

            {
                "metadata": {
                    "ad_data": null
                },
                "id": "a9f4f694-09b0-47c0-be5e-6a8b0032a08e",
                "slug": "lost-2004",
                "title": "Lost",
                "overview": "The survivors of a plane crash are forced to work together in order to survive on a seemingly deserted tropical island.",
                "reelgood_synopsis": "Lost featuring Naveen Andrews and Matthew Fox has one or more episodes streaming with subscription on Hulu, free on Freevee (Via Prime Video), available for purchase on Google Play, and 2 others. It's an action & adventure and drama show with 124 episodes over 6 seasons. Lost is no longer running and has no plans to air new episodes or seasons. It has a high IMDb audience rating of 8.3 (563,917 votes) and was well received by critics.",
                "trailer": {
                    "site": "youtube",
                    "key": "KTu8iDynwNc",
                    "url": null
                },
                "trailers": [
                    {
                        "site": "youtube",
                        "key": "KTu8iDynwNc",
                        "url": null
                    }
                ],
                "imdb_rating": 8.3,
                "rt_critics_rating": null,
                "rt_audience_rating": 91,
                "has_poster": true,
                "has_backdrop": true,
                "season_count": 6,
                "classification": "16+",
                "last_modified_at": "2023-03-16T00:00:00",
                ...
            }
        >>> getTvShowApi('um texto sem correspondência')

            {
                "type": "https://tools.ietf.org/html/rfc7231#section-6.5.4",
                "title": "Not Found",
                "status": 404,
                "traceId": "00-cfa2d680394c5d6dcba63ceda93993eb-b2dc38cd80923d52-00"
            }
    """
    result = requests.get(f"https://api.reelgood.com/v3.0/content/show/{id}")
    if result.status_code == requests.codes.ok:
        return Result(value=result.text)
    else:
        return Result(error=Exception(result.content))


def get_person_details(id) -> Result:
    result = requests.get(
        f"https://api.reelgood.com/v3.0/content/people/{id}?availability=all&content_kind=both&hide_seen=true&hide_tracked=true&hide_watchlisted=true&imdb_end=10&imdb_start=0&region=us&rg_end=100&rg_start=0&skip=0&sort=0&take=35&year_end=2023&year_start=1900"
    )
    if result.status_code == requests.codes.ok:
        return Result(value=result.text)
    else:
        return Result(error=Exception(result.content))



class Creative:
    id: int
    campaign_name: str
    campaign_id: int
    advertiser_id: int
    ad_creative_type_id: int
    source_id: str
    image_override: None
    external_data: None
    links: None
    on_services: bool

    def __init__(
        self,
        id: int,
        campaign_name: str,
        campaign_id: int,
        advertiser_id: int,
        ad_creative_type_id: int,
        source_id: str,
        image_override: None,
        external_data: None,
        links: None,
        on_services: bool,
    ) -> None:
        self.id = id
        self.campaign_name = campaign_name
        self.campaign_id = campaign_id
        self.advertiser_id = advertiser_id
        self.ad_creative_type_id = ad_creative_type_id
        self.source_id = source_id
        self.image_override = image_override
        self.external_data = external_data
        self.links = links
        self.on_services = on_services



class RegionalAvailability:
    it: list[int]
    nl: list[int]
    de: list[int]
    fr: list[int]
    es: list[int]
    gb: list[int]
    au: list[int]
    us: list[int]

    def __init__(
        self,
        it: list[int],
        nl: list[int],
        de: list[int],
        fr: list[int],
        es: list[int],
        gb: list[int],
        au: list[int],
        us: list[int],
    ) -> None:
        self.it = it
        self.nl = nl
        self.de = de
        self.fr = fr
        self.es = es
        self.gb = gb
        self.au = au
        self.us = us



from rich.console import Console
from rich.table import Table, box

from popcorn.api.models.PersonApi import PersonApi
from popcorn.tables.people import people
from popcorn.tables.sources import source
from popcorn.tables.TableInterface import TableInterface
from popcorn.utils import format_date, format_date_str

topTable = [
    ":scroll: Overview",
    ":date: Release",
    ":watch: Time",
    ":100: IMDB",
    ":traffic_light: Classification",
    ":movie_camera: Trailers",
]


def __initTable__() -> Table:
    table = Table(
        highlight=True,
        show_header=True,
        show_edge=True,
        expand=True,
        show_lines=True,
        box=box.DOUBLE_EDGE,
    )
    for title in topTable:
        table.add_column(title)

    return table


def table_details(item: TableInterface) -> Table:
    table = __tableItem__(
        table=__initTable__(),
        overview=item.get_overview(),
        release=item.get_date(),
        time=item.get_time(),
        imdb=item.get_imdb(),
        classification=item.get_classification(),
        trailers=item.get_trailers(),
    )
    return table


def __tableItem__(
    table: Table,
    overview: str,
    release: str,
    time: str,
    imdb: float,
    classification: str,
    trailers: list[str],
) -> Table:
    table.add_row(
        overview, release, time, imdb, classification, trailers, end_section=True
    )
    return table



class Season:
    def __init__(
        self,
        id,
        number,
        has_poster,
        aired_at,
        completed_at,
        episodes_unwatched,
        availability,
        episodes,
    ):
        self.id = id
        self.number = number
        self.has_poster = has_poster
        self.aired_at = aired_at
        self.completed_at = completed_at
        self.episodes_unwatched = episodes_unwatched
        self.availability = availability
        self.episodes = episodes


class Availability:
    def __init__(self, purchase_available, sources):
        self.purchase_available = purchase_available
        self.sources = sources


class Source:
    def __init__(self, source_name, access_type):
        self.source_name = source_name
        self.access_type = access_type



class ReelgoodScores:
    streamability: float
    content: float
    follow_through: None
    reelgood_rank: int
    reelgood_popularity: float

    def __init__(
        self,
        streamability: float,
        content: float,
        follow_through: None,
        reelgood_rank: int,
        reelgood_popularity: float,
    ) -> None:
        self.streamability = streamability
        self.content = content
        self.follow_through = follow_through
        self.reelgood_rank = reelgood_rank
        self.reelgood_popularity = reelgood_popularity



import json

from popcorn.api.api import get_movie_api, get_person_details, get_tvshow_api, search
from popcorn.api.models.PersonApi import PersonApi
from popcorn.api.models.Result import Result
from popcorn.api.models.SearchApi import ContentType
from popcorn.dto import format_list
from popcorn.models.ItemMovie import ItemMovie
from popcorn.models.ItemShow import ItemShow
from popcorn.models.Search import Search
from popcorn.utils import format_date


def search_reel(query: str, year: int | None, type: str | None) -> list[Search]:
    result: Result = search(query=query)
    if result.error is None:
        objJson = json.loads(result.value)
        if objJson["items"].__len__() == 0:
            return []
        listSearch = filters_args(objJson["items"], year, type)
        return format_list(listSearch)
    else:
        []


def person_reel(id) -> Result:
    result = get_person_details(id)
    if result.error is None:
        return Result(value=PersonApi(**json.loads(result.value)))
    else:
        return Result(Exception("Match not Found..."))


def transform_item(slug: str, type: str) -> Result:
    match type:
        case ContentType.M.value:
            movie = get_movie_api(slug)
            if movie.error is None:
                return Result(value=ItemMovie(**json.loads(movie.value)))
            else:
                return Result(Exception("Match not Found..."))
        case ContentType.S.value:
            tv = get_tvshow_api(slug)
            if tv.error is None:
                return Result(value=ItemShow(**json.loads(tv.value)))
            else:
                return Result(Exception("Match not Found..."))
        case _:
            return Result(Exception("Match not Found..."))


def filter_by_year(item, year) -> bool:
    if item["released_on"] == None:
        return True
    return format_date(item["released_on"]).year == year


def filter_by_type(item, type) -> bool:
    if item["content_type"] == None:
        return True
    return item["content_type"] == type


def filters_args(items: list[any], year: int | None, type: str | None):
    if year == None and type == None:
        return items
    if year != None and type == None:
        return list(filter(lambda x: (filter_by_year(x, year)), items))
    if year == None and type != None:
        return list(filter(lambda x: (filter_by_type(x, type)), items))
    if year != None and type != None:
        return list(
            filter(
                lambda x: (filter_by_type(x, type) and filter_by_year(x, year)), items
            )
        )



from dataclasses import dataclass

from popcorn.utils import create_url as fullUrl
from popcorn.utils import format_date, format_sources, format_type


@dataclass
class Search:
    id: int
    title: str
    release: str
    type: str
    imdb: float
    online: list[str]
    slug: str

    def format_long_type(self):
        return format_type(self.type)

    def format_online(self):
        return format_sources(self.online)

    def create_url(self):
        return fullUrl(self.type, self.slug)

    def format_simple_date(self):
        result = format_date(self.release)
        if result == None:
            return "----"
        return result.year

    def format_simple_date_str(self):
        return str(self.format_simple_date())

    def imdb_str(self) -> str:
        if self.imdb != None:
            return str(self.imdb)
        else:
            return "-.-"



from popcorn.models.Content import Content
from popcorn.models.Streamability import Streamability


class ScoreBreakdown:
    streamability: list[Streamability]
    content: Content

    def __init__(self, streamability: list[Streamability], content: Content) -> None:
        self.streamability = streamability
        self.content = content



from datetime import datetime
from typing import Optional

from popcorn.models.Ios import Ios
from popcorn.models.Links import Links
from popcorn.models.Metadata import Metadata
from popcorn.models.Person import Person
from popcorn.models.RatingStats import RatingStats
from popcorn.models.ReelgoodScores import ReelgoodScores
from popcorn.models.RegionalAvailability import RegionalAvailability
from popcorn.models.ScoreBreakdown import ScoreBreakdown
from popcorn.models.SourceAdPlacement import SourceAdPlacement
from popcorn.models.Tag import Tag
from popcorn.models.Trailer import Trailer
from popcorn.tables.TableInterface import TableInterface
from popcorn.utils import create_url as full_urls
from popcorn.utils import filterTrailerByService, format_date
from popcorn.utils import format_sources as format_sources_util
from popcorn.utils import format_trailers


class References:
    web: Ios
    ios: Ios
    android: Optional[Ios]
    webos: Optional[Ios]

    def __init__(
        self, web: Ios, ios: Ios, android: Optional[Ios], webos: Optional[Ios]
    ) -> None:
        self.web = web
        self.ios = ios
        self.android = android
        self.webos = webos


class SourceData:
    links: Links
    references: Optional[References]
    web_link: str

    def __init__(
        self, links: Links, references: Optional[References], web_link: str
    ) -> None:
        self.links = links
        self.references = references
        self.web_link = web_link


class Availability:
    rental_cost_hd: Optional[float | None]
    rental_cost_sd: Optional[float | None]
    purchase_cost_hd: Optional[float | None]
    purchase_cost_sd: Optional[float | None]
    source_id: str
    source_name: str
    access_type: int
    source_data: SourceData

    def __init__(
        self,
        rental_cost_hd: Optional[float | None],
        rental_cost_sd: Optional[float | None],
        purchase_cost_hd: Optional[float | None],
        purchase_cost_sd: Optional[float | None],
        source_id: str,
        source_name: str,
        access_type: int,
        source_data: SourceData,
    ) -> None:
        self.rental_cost_hd = rental_cost_hd
        self.rental_cost_sd = rental_cost_sd
        self.purchase_cost_hd = purchase_cost_hd
        self.purchase_cost_sd = purchase_cost_sd
        self.source_id = source_id
        self.source_name = source_name
        self.access_type = access_type
        self.source_data = source_data


class TextReasons:
    text: str


class ItemMovie(TableInterface):
    metadata: Metadata
    id: str
    slug: str
    title: str
    overview: str
    tagline: str
    reelgood_synopsis: str
    classification: str
    runtime: int
    released_on: datetime
    trailer: Trailer
    trailers: list[Trailer]
    language: str
    has_poster: bool
    has_backdrop: bool
    imdb_rating: float
    imdb_votes: int
    rt_critics_rating: None
    rt_audience_rating: int
    last_modified_at: datetime
    user_rating: None
    watchlisted: bool
    seen: bool
    user_lists: None
    sources: list[str]
    on_free: bool
    on_rent_purchase: bool
    genres: list[int]
    tags: list[Tag]
    countries: list[str]
    availability: list[Availability]
    people: list[Person]
    score_breakdown: ScoreBreakdown
    reelgood_scores: ReelgoodScores
    regional_availability: RegionalAvailability
    source_ad_placement: SourceAdPlacement
    source_ad_placements: list[SourceAdPlacement]
    rating_stats: RatingStats
    reviews_count: int
    madlib_synopsis: str
    where_to_wathc: str
    reasons_to_watch: list[TextReasons]

    def __init__(
        self,
        metadata: Metadata,
        id: str,
        slug: str,
        title: str,
        overview: str,
        tagline: str,
        reelgood_synopsis: str,
        classification: str,
        runtime: int,
        released_on: datetime,
        trailer: Trailer,
        trailers: list[Trailer],
        language: str,
        has_poster: bool,
        has_backdrop: bool,
        imdb_rating: float,
        imdb_votes: int,
        rt_critics_rating: None,
        rt_audience_rating: int,
        last_modified_at: datetime,
        user_rating: None,
        watchlisted: bool,
        seen: bool,
        user_lists: None,
        sources: list[str],
        on_free: bool,
        on_rent_purchase: bool,
        genres: list[int],
        tags: list[Tag],
        countries: list[str],
        availability: list[Availability],
        people: list[Person],
        score_breakdown: ScoreBreakdown,
        reelgood_scores: ReelgoodScores,
        regional_availability: RegionalAvailability,
        source_ad_placement: SourceAdPlacement,
        source_ad_placements: list[SourceAdPlacement],
        rating_stats: RatingStats,
        reviews_count: int = 0,
        madlib_synopsis: str = "",
        where_to_watch: str = "",
        reasons_to_watch: list[TextReasons] = [],
        content_blocks: any = "",
    ) -> None:
        self.metadata = metadata
        self.id = id
        self.slug = slug
        self.title = title
        self.overview = overview
        self.tagline = tagline
        self.reelgood_synopsis = reelgood_synopsis
        self.classification = classification
        self.runtime = runtime
        self.released_on = released_on
        self.trailer = trailer
        self.trailers = trailers
        self.language = language
        self.has_poster = has_poster
        self.has_backdrop = has_backdrop
        self.imdb_rating = imdb_rating
        self.imdb_votes = imdb_votes
        self.rt_critics_rating = rt_critics_rating
        self.rt_audience_rating = rt_audience_rating
        self.last_modified_at = last_modified_at
        self.user_rating = user_rating
        self.watchlisted = watchlisted
        self.seen = seen
        self.user_lists = user_lists
        self.sources = sources
        self.on_free = on_free
        self.on_rent_purchase = on_rent_purchase
        self.genres = genres
        self.tags = tags
        self.countries = countries
        self.availability = availability
        self.people = people
        self.score_breakdown = score_breakdown
        self.reelgood_scores = reelgood_scores
        self.regional_availability = regional_availability
        self.source_ad_placement = source_ad_placement
        self.source_ad_placements = source_ad_placements
        self.rating_stats = rating_stats
        self.reviews_count = reviews_count
        self.madlib_synopsis = madlib_synopsis
        self.where_to_watch = where_to_watch
        self.reasons_to_watch = reasons_to_watch
        self.content_blocks = content_blocks

    def get_number_seasons(self):
        return "0"

    def get_overview(self):
        return self.overview

    def get_date(self) -> str:
        return self.format_date()

    def get_time(self) -> str:
        return self.format_time()

    def get_imdb(self) -> str:
        return self.imdb_str()

    def get_classification(self) -> str:
        return (
            f"[b][white]{self.classification}" if self.classification else "Who knows"
        )

    def get_trailers(self) -> str:
        return self.get_list_trailers()

    def format_date(self) -> str:
        return (
            str(format_date(self.released_on).date())
            if self.released_on is not None
            else "-- --"
        )

    def format_time(self) -> str:
        if self.runtime is not None:
            hours, minutes = divmod(self.runtime, 60)
            return f"{hours}h {minutes}m"
        else:
            return "-- --"

    def get_list_trailers(self) -> str:
        if self.trailer is not None:
            self.trailers.append(self.trailer)
            return format_trailers(filterTrailerByService(self.trailers))
        else:
            search = self.slug.replace("-", "+")
            return f"[bold blue][link=https://www.youtube.com/results?search_query={search}]Search Youtube[/link]"

    def format_sources(self) -> str:
        values = list(map(lambda x: x["source_name"], self.availability))
        return format_sources_util(values)

    def create_url(self):
        return full_urls("m", self.slug)

    def imdb_str(self) -> str:
        if self.imdb_rating != None:
            return str(self.imdb_rating)
        else:
            return "-.-"



from datetime import datetime
from typing import List, Optional


class Person:
    id: int
    slug: str
    name: str
    birthdate: Optional[datetime]
    has_poster: bool
    has_square: bool
    role_type: int
    role: Optional[str | None]
    rank: Optional[int]

    def __init__(
        self,
        id: int,
        slug: str,
        name: str,
        birthdate: Optional[datetime],
        has_poster: bool,
        has_square: bool,
        role_type: int,
        role: Optional[str],
        rank: Optional[int],
    ) -> None:
        self.id = id
        self.slug = slug
        self.name = name
        self.birthdate = birthdate
        self.has_poster = has_poster
        self.has_square = has_square
        self.role_type = role_type
        self.role = role
        self.rank = rank



from popcorn.models.ListingKey import ListingKey


class Rank:
    rank: int
    text: str
    listing_key: ListingKey

    def __init__(self, rank: int, text: str, listing_key: ListingKey) -> None:
        self.rank = rank
        self.text = text
        self.listing_key = listing_key



from rich.console import Console
from rich.table import Table

from popcorn.models.Search import Search

console = Console()

searchType = [
    ":id: Index",
    ":blue_book: Title",
    ":date: Release",
    ":flags: Type",
    ":100: IMDB rate",
    ":movie_camera: Online",
    ":link: Details",
]


def __init_table__() -> Table:
    table = Table(
        title="POPCORN SHOW - SEARCH",
        highlight=True,
        show_header=True,
        show_edge=True,
        expand=True,
    )
    for title in searchType:
        table.add_column(title)
    return table


def table_search(items: list[Search]) -> Table:
    table = __init_table__()
    for index, item in enumerate(items):
        __tableItem__(
            table=table,
            id=str(index + 1),
            title=f"[white]{item.title}[/white]",
            release=item.format_simple_date_str(),
            type=item.format_long_type(),
            imdb=item.imdb_str(),
            online=item.format_online(),
            url=item.create_url(),
        )
    return table


def __tableItem__(
    table: Table,
    id: int,
    title: str,
    imdb: float,
    type: chr,
    release: str,
    online: str,
    url: str,
):
    table.add_row(str(id), title, release, type, imdb, online, url)



class AlsoWatched:
    id: int
    content_type: str
    slug: str
    title: str

    def __init__(self, id: int, content_type: str, slug: str, title: str) -> None:
        self.id = id
        self.content_type = content_type
        self.slug = slug
        self.title = title



class TableInterface:
    def get_number_seasons() -> str:
        pass

    def get_overview() -> str:
        pass

    def get_date() -> str:
        pass

    def get_time() -> str:
        pass

    def get_imdb() -> str:
        pass

    def get_classification() -> str:
        pass

    def get_trailers() -> str:
        pass



from rich.console import Console
from rich.table import Table, box

from popcorn.tables.people import people
from popcorn.tables.sources import source
from popcorn.tables.TableInterface import TableInterface

console = Console()

topTable = [
    ":scroll: Overview",
    ":date: Release",
    ":watch: Season",
    ":100: IMDB",
    ":traffic_light: Classification",
    ":movie_camera: Trailers",
]


table = Table(
    highlight=True,
    show_header=True,
    show_edge=True,
    expand=True,
    show_lines=True,
    box=box.DOUBLE_EDGE,
)


def table_details_show(item: TableInterface) -> Table:
    for title in topTable:
        table.add_column(title)

    __tableItem(
        overview=item.get_overview(),
        release=item.get_date(),
        seasons=item.get_number_seasons(),
        imdb=item.get_imdb(),
        classification=item.get_classification(),
        trailers=item.get_trailers(),
    )
    return table


def __tableItem(
    overview: str,
    release: str,
    seasons: str,
    imdb: float,
    classification: str,
    trailers: list[str],
):
    table.add_row(
        overview, release, seasons, imdb, classification, trailers, end_section=True
    )



from dataclasses import dataclass
from typing import Optional

from popcorn.api.models.SearchApi import Item


@dataclass
class PersonApi:
    no_index: bool
    id: str
    slug: str
    name: str
    birthplace: str | None
    birthdate: Optional[str]
    deathdate: Optional[str]
    gender: str
    biography: str | None
    homepage: Optional[str]
    has_poster: bool
    has_square: bool
    initial_credits: list[Item]
